(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007                                                    *)
(*    CEA (Commissariat  l'nergie Atomique)                             *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

open Cilutil

let do_project () =
  let result = ref [] in
  let first = ref true in
  Project.iter_on_projects (fun p ->
                              let f = !first in
                              first := false;
                              result := (p,f)::!result);
  List.sort
    (fun (p1,_) (p2,_) -> compare (Project.name p1) (Project.name p2))
    !result

let make_project_entries window =
(*  let actions = window#actions in*)
  let actions = GAction.action_group ~name:"project_manager" () in
  let ui = window#ui_manager in
  ui#insert_action_group actions 0;
  let merge_id = ref None in
  let demerge _acc =
    ui#remove_ui (out_some !merge_id);
    ui#remove_action_group actions;
    ui#ensure_update()
  in
  let switcher = do_project () in
  let make_action (project,current) =
    let name = Project.name project in
    (GAction.add_action
      ~label:(Pretty_utils.escape_underscores name)
      ~callback:(fun acc ->
                   Project.set_current project;
                   demerge acc;
                   window#reset ())
      ~tooltip:""
      ~accel:""
      ~stock:(if current then `MEDIA_PLAY
              else `MEDIA_PAUSE)
      name
      :unit GAction.entry)
  in
  GAction.add_actions actions
    (List.map make_action switcher);
  let location =
    Format.sprintf
      "<ui><menubar name='MenuBar'>
          <menu action='ProjectMenu'>
          %s
          </menu>
          </menubar></ui>"
      (List.fold_right
         (fun (p,_) acc ->
            (Format.sprintf "<menuitem action='%s'/>"
              (Project.name p)) ^ acc)
         switcher
         "")
  in
  merge_id := Some (ui#add_ui_from_string location);
  ui#ensure_update()

(** Register this dialog in main window menu bar *)
let () =
  Design.register_extension
    (fun window ->
       GAction.add_actions window#actions
         [GAction.add_action
            "ProjectMenu"
            ~label:"_Change project"
            ~callback:(fun _ -> make_project_entries window;
                         window#ui_manager#ensure_update())];
       (window#actions#get_action "ProjectMenu")#set_hide_if_empty false;
       let location =
         "<ui><menubar name='MenuBar'>
          <menu action='ProjectMenu'>
          </menu>
          </menubar></ui>"
       in
       ignore (window#ui_manager#add_ui_from_string location))

(*
Local Variables:
compile-command: "LC_ALL=C make -C ../.. -j"
End:
*)
