(* $Id: register.ml,v 1.7 2008/03/06 08:23:29 uid568 Exp $ *)

open Cil_types
open Cilutil

module Occurrences: sig
  val add: varinfo -> kinstr -> lval -> unit
  val get: varinfo -> (kinstr * lval) list
  val self: Project.Computation.t
end = struct

  module M = 
    Kernel_computation.VarinfoHashtbl
      (Datatype.List
	 (Datatype.Couple(Kernel_datatype.Kinstr)(Kernel_datatype.Lval)))
      (struct 
	 let size = 17 
	 let name = Project.Computation.Name.make "occurrences"
	 let dependencies = [ Db.Value.self ]
       end)

  let add vi ki lv =
    let elt = ki, lv in
    ignore (M.memo ~change:(fun l -> elt :: l) (fun _ -> [ elt ]) vi)

  let get vi = try M.find vi with Not_found -> []

  let self = M.self

end

class occurrence = object (self)

  inherit Visitor.generic_frama_c_visitor 
    (Cil.inplace_visit ())
    (Project.current ())

  val mutable decls = []

  method private current_ki =
    match self#current_stmt with None -> Kglobal | Some s -> Kstmt s
      
  method vvdec vi =
    let ki = self#current_ki in
    if Db.Value.is_accessible ki then begin
      let z = 
	!Db.Value.lval_to_zone 
	  ki ~with_alarms:CilE.warn_none_mode (Var vi, NoOffset) 
      in
      decls <-  (vi, z) :: decls
    end;
    Cil.DoChildren

  method vlval lv = 
    let ki = self#current_ki in
    if Db.Value.is_accessible ki then begin
      let z = !Db.Value.lval_to_zone ki ~with_alarms:CilE.warn_none_mode lv in
      if not (Locations.Zone.equal Locations.Zone.bottom z) then 
        List.iter 
          (fun (vi, zvi) -> 
	     if Locations.Zone.intersects z zvi then Occurrences.add vi ki lv)
          decls
    end;
    Cil.DoChildren


  method vterm_lval tlv = 
    (try
       let lv = !Db.Properties.Interp.term_lval_to_lval tlv in
       ignore (self#vlval lv)
     with Invalid_argument msg ->
       if Cmdline.Debug.get () > 0 then Format.printf "[occurrence:] %s@." msg);
    Cil.DoChildren

  initializer !Db.Value.compute ()

end

let compute, _self =
  let run () =
    ignore 
      (Cil.visitCilFile (new occurrence :> Cil.cilVisitor) (Cil_state.file ()))
  in
  Computation.apply_once 
    (Project.Computation.Name.make "Occurrence.compute") 
    [ Occurrences.self ]
    run

let get vi =
  compute ();
  Occurrences.get vi
      
let () = Db.Occurrence.get := get

(*
Local Variables:
compile-command: "LC_ALL=C make -C ../.. -j"
End:
*)
